/*
 DirectionService - controll writting direction
 Copyright (C) 2003 Nir Soffer

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 details: http://www.gnu.org/licenses/gpl.html
 */



#import "DirectionService.h"

@implementation DirectionService


- (void)startDirectionService:(NSPasteboard *)pboard
                     userData:(NSString *)data
                        error:(NSString **)error
/* Put a Unicode RLE or LRE character into to the pasteboard */ 
{
    NSString *beginDirectionString;

    // check userData and setup direction strings
    if ([data isEqualToString:@"RTL"]) { // Right to Left service
        beginDirectionString = [self stringWithUnicodeCharacterNumber:0x202B]; // Unicode RLE
    }
    else if ([data isEqualToString:@"LTR"]) { // Left to Right service
        beginDirectionString = [self stringWithUnicodeCharacterNumber:0x202A]; // Unicode LRE
    }
    else { // bad data
        NSLog(@"Bad userData, probably error in info.plist");
        return;
    }

    [self putString:beginDirectionString onPasteboard:pboard];
}


- (void)endDirectionService:(NSPasteboard *)pboard
                   userData:(NSString *)data
                      error:(NSString **)error
    /* Put a Unicode PDF character into to the pasteboard */
{
    NSString *endDirectionString;

    endDirectionString =
        [self stringWithUnicodeCharacterNumber:0x202C]; // Unicode PDF    
    [self putString:endDirectionString onPasteboard:pboard];
}


- (void)directionService:(NSPasteboard *)pboard
                userData:(NSString *)data
                   error:(NSString **)error
/*
 This service _should_ take a string from the pasteboard and return the string
 with direction characters inserted in each paragraph.

 This early version only enclose the selection with direction characters, so
 the user should select one paragraph for each service call.
*/
{
    NSString *origString; // original string from the pasteboard
    NSString *beginDirectionString; // begin a direction - Unicode RLE or LRE
    NSString *endDirectionString; // end the direction - Unicode PDF
    NSMutableString *newString; // new string to put back on the pasteboard
    
    // get a string from the pasteboard
    origString = [self getStringFromPasteboard:pboard];
    if (origString == nil) {
        *error = NSLocalizedString(@"Error: Pasteboard doesn't contain a string.",
            @"Pasteboard couldn't give string.");
        return;
    }
    
    // check userData and setup direction strings
    if ([data isEqualToString:@"RTL"]) { // Right to Left service
        beginDirectionString =
            [self stringWithUnicodeCharacterNumber:0x202B]; // Unicode RLE
    }
    else if ([data isEqualToString:@"LTR"]) { // Left to Right service
        beginDirectionString =
            [self stringWithUnicodeCharacterNumber:0x202A]; // Unicode LRE
    }
    else { // bad data
        NSLog(@"Bad userData, probably error in info.plist");
        return;
    }

    // setup strings
    endDirectionString = [self stringWithUnicodeCharacterNumber:0x202C]; // Unicode PDF
       

    // create the new string and insert the direction marks
    // this should be done for each paragraph
    newString = [NSMutableString stringWithString:origString];
    [newString insertString:beginDirectionString atIndex:0];
    [newString insertString:endDirectionString atIndex:[newString length]];
    
    // Finally, return the string to the pasteboard
    // Is it ok to return a muatable string instead of string? - seems to work
    [self putString:newString onPasteboard:pboard];
}


- (NSString *)stringWithUnicodeCharacterNumber:(unichar)characterNumber
/* utility method to create strings from Unicode characters code */
{
    unichar unicodeCharacter = characterNumber;
    return [NSString stringWithCharacters:&unicodeCharacter length:1];
}


- (NSString *)getStringFromPasteboard:(NSPasteboard *)pb
/* Utility method to return a string from the pasteboard nil */
{
    NSArray *pbTypes; // types of objects on the pasteboard
    
    pbTypes = [pb types];
    if ([pbTypes containsObject:NSStringPboardType]) 
    {
        return [pb stringForType:NSStringPboardType];
    }
    
    return nil; // no soup for us on the pasteboard
}


- (void)putString:(NSString *)aString onPasteboard:(NSPasteboard *)pb
/* utility method to put a string on the pastebaord */
{
    NSArray *types; // types of objects to put on the pasteboard
    
    types = [NSArray arrayWithObject:NSStringPboardType];
    [pb declareTypes:types owner:nil];
    [pb setString:aString forType:NSStringPboardType];
}


@end
